/**
 * Copyright (c) 2011, REMPL.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution.
 * 3) Neither the name of the REMPL.com nor the names of its
 * contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.rempl.reporters.pdd;

// for logging and error handling
import com.ymock.util.Logger;
import com.ymock.util.RuntimeProblem;

// file management
import java.io.File;

// API for XML transformations
// Implementation class name is defined in META-INF/services
import javax.xml.transform.Source;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

/**
 * Transforms XML into XHTML, via XSLT.
 *
 * @author Yegor Bugayenko (yegor@rempl.com)
 * @version $Id: Transformer.java 1614 2011-07-10 19:05:43Z guard $
 */
final class Transformer {

    /**
     * DOM transformer factory, DOM.
     */
    private static TransformerFactory transFactory =
        TransformerFactory.newInstance();

    /**
     * Perform XSLT translation of the XML file.
     * @param xml The XML document to be translated
     * @param html The HTML document to be created
     * @see #report(Model, File)
     * @throws java.io.IOException If some I/O problem occurs
     * @todo #18 Would be nice to validate the output XHTML produced
     *           against pre-defined XML Schema.
     */
    public void transform(final File xml, final File html)
        throws java.io.IOException {
        if (!html.exists() && html.createNewFile()) {
            Logger.info(
                this,
                "#transform(%s, %s): new XHTML file created",
                xml.getPath(),
                html.getPath()
            );
        }
        final Source xsl = new StreamSource(
            this.getClass().getResourceAsStream("/pdd.xsl")
        );
        try {
            this.transFactory.newTransformer(xsl).transform(
                new StreamSource(xml),
                new StreamResult(html)
            );
        } catch (javax.xml.transform.TransformerConfigurationException ex) {
            throw RuntimeProblem.make(ex);
        } catch (javax.xml.transform.TransformerException ex) {
            throw RuntimeProblem.make(ex);
        }
        Logger.info(
            this,
            "#transform(%s, %s): XHTML generated (%d bytes)"
            + " through XSLT translation",
            xml.getPath(),
            html.getPath(),
            html.length()
        );
    }

}
