/**
 * Copyright (c) 2011, REMPL.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution.
 * 3) Neither the name of the REMPL.com nor the names of its
 * contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.rempl.reporters.pdd;

// REMPL API from com.rempl:rempl-api classes
import com.rempl.api.Element;
import com.rempl.api.Model;
import com.ymock.util.RuntimeProblem;

// date processing
import java.text.SimpleDateFormat;

// collection management
import java.util.Collection;
import java.util.Date;
import java.util.Locale;

/**
 * One issue, from UML Model.
 *
 * @author Yegor Bugayenko (yegor@rempl.com)
 * @version $Id: UmlIssue.java 1614 2011-07-10 19:05:43Z guard $
 * @see PDDReporter#report(Model, File)
 */
final class UmlIssue implements Issue {

    /**
     * UML field where TIME of change is kept.
     */
    private static final String FIELD_TIME = "time";

    /**
     * UML field where VALUE of change is kept.
     */
    private static final String FIELD_VALUE = "value";

    /**
     * Name of the issue.
     */
    private final String name;

    /**
     * Title of the issue.
     */
    private final String title;

    /**
     * Name of the owner/assignee.
     */
    private final String assignee;

    /**
     * Public ctor.
     * @param element The element from the model
     * @param model The model itself
     * @see PDDReporter#report(Model, File)
     */
    public UmlIssue(final Element element, final Model model) {
        this.name = element.get("name");
        this.title = this.latest("title", element, model)
            .get(this.FIELD_VALUE);
        this.assignee = this.person(
            this.latest("assignee", element, model),
            model
        ).get("email");
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getName() {
        return this.name;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getTitle() {
        return this.title;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getAssignee() {
        return this.assignee;
    }

    /**
     * Retrieve the latest field from the issue.
     * @param field The name of the field
     * @param element The element from the model
     * @param model The model itself
     * @return The value of the field
     * @see #Issue(Element, Model)
     * @todo #18 This implementation is temporary. We should use OCL
     *       here, as soon as it works in rempl-api.
     */
    private Element latest(final String field, final Element element,
        final Model model) {
        final Collection<Element> changes =
            model.query("context IssueChange inv: true");
        Element found = null;
        for (Element change : changes) {
            if (!change.refsTo("relatedIssue", element)) {
                continue;
            }
            if (!change.get("component").equals(field)) {
                continue;
            }
            if (found == null) {
                found = change;
                continue;
            }
            final boolean newer = this.date(found.get(this.FIELD_TIME))
                .before(this.date(change.get(this.FIELD_TIME)));
            if (newer) {
                found = change;
            }
        }
        return found;
    }

    /**
     * Parse date from ITS UML code.
     * @param txt The text from UML
     * @return The date
     * @see latest(String,Element,Model)
     */
    private Date date(final String txt) {
        try {
            return new SimpleDateFormat(
                "yyyy-MM-dd'T'HH:mm:ssz",
                Locale.ENGLISH
            ).parse(txt);
        } catch (java.text.ParseException ex) {
            throw RuntimeProblem.make(ex);
        }
    }

    /**
     * Get person name by this change.
     * @param change The IssueChange UML element
     * @param model The model to work with
     * @return The person
     * @see UmlIssue(Element,Model)
     */
    private Element person(final Element change, final Model model) {
        final Collection<Element> people =
            model.query("context Person inv: true");
        for (Element person : people) {
            if (change.refsTo(this.FIELD_VALUE, person)) {
                return person;
            }
        }
        // this should never happen
        throw new IllegalStateException("Failed to find owner of issue");
    }

}
