/**
 * Copyright (c) 2011, REMPL.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution.
 * 3) Neither the name of the REMPL.com nor the names of its
 * contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.rempl.reporters.pdd;

// Bridge API from com.rempl.readers.its:bridge-its-api
import com.rempl.readers.its.bridge.CodeUrlBuilder;
import com.rempl.readers.its.bridge.IssueUrlBuilder;

// for logging and error handling
import com.ymock.util.Logger;
import com.ymock.util.RuntimeProblem;

// for XML marshalling
import java.io.StringWriter;
import java.io.Writer;

// regular expressions
import java.util.regex.Matcher;
import java.util.regex.Pattern;

// JAXB API
// Implementation class name is defined in META-INF/services
import javax.xml.bind.JAXBContext;
import javax.xml.bind.Marshaller;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

/**
 * Work Breakdown Structure (WBS).
 *
 * @author Yegor Bugayenko (yegor@rempl.com)
 * @version $Id: Wbs.java 1614 2011-07-10 19:05:43Z guard $
 * @see PDDReporter#report(Model, File)
 */
@XmlRootElement(name = "pdd")
public final class Wbs {

    /**
     * Prefix of PDD markers.
     */
    private static final String PREFIX = "PDD";

    /**
     * Separator inside PDD marker.
     */
    private static final String SEPARATOR = "-";

    /**
     * The builder for creating issue URLs.
     */
    private IssueUrlBuilder issueUrlBuilder;

    /**
     * The builder for creating code URLs.
     */
    private CodeUrlBuilder codeUrlBuilder;

    /**
     * Parent work package.
     */
    private WorkPackage sud = new WorkPackage();

    /**
     * Public ctor.
     */
    public Wbs() {
        this.issueUrlBuilder = null;
        this.codeUrlBuilder = null;
    }

    /**
     * Set a builder for issue URLs.
     * @param builder The builder to use
     * @see PDDReporter#report(Model, File)
     */
    public void setIssueUrlBuilder(final IssueUrlBuilder builder) {
        this.issueUrlBuilder = builder;
        Logger.debug(
            this,
            "#setIssueUrlBuilder(%s): configured",
            builder.getClass().getName()
        );
    }

    /**
     * Set a builder for code URLs.
     * @param builder The builder to use
     * @see PDDReporter#report(Model, File)
     */
    public void setCodeUrlBuilder(final CodeUrlBuilder builder) {
        this.codeUrlBuilder = builder;
        Logger.debug(
            this,
            "#setCodeUrlBuilder(%s): configured",
            builder.getClass().getName()
        );
    }

    /**
     * Add new issue.
     * @param issue The issue
     * @see PDDReporter#report(Model, File)
     * @todo #2:1 We should add assignee information to the issue
     *            as well, not only to the puzzle beneath it.
     * @todo #2:1 Issue title should be placed as a subject to
     *            the workpackage.
     */
    public void add(final Issue issue) {
        if (this.issueUrlBuilder == null) {
            throw new IllegalStateException("Call setIssueUrlBuilder() first");
        }
        final Pattern pattern = Pattern.compile(
            // everything before the marker, and start of group
            ".*("
            + this.PREFIX
            + this.SEPARATOR
            + "([0-9A-F]+)"
            + this.SEPARATOR
            + "([0-9A-Z-]+)"
            // end of group, and everything after the marker
            + ").*"
        );
        final Matcher matcher = pattern.matcher(issue.getTitle());
        if (matcher.matches()) {
            final String marker = matcher.group(1);
            this.sud
                // find the UP-UP-LINK of this issue
                // @checkstyle MagicNumber (1 line)
                .locate(matcher.group(3))
                // find a puzzle under this UP-UP-LINK
                .locate(marker)
                // set issue name into it and other config
                .setIssue(
                    new WorkPackage.XmlIssue(
                        issue.getName(),
                        this.issueUrlBuilder.build(issue.getName()),
                        issue.getAssignee()
                    )
                );
            Logger.debug(
                this,
                "#add(%s): issue %s injected into WBS, marker: '%s'",
                issue.getClass().getName(),
                issue.getName(),
                marker
            );
        }
    }

    /**
     * Add new puzzle.
     * @param puzzle The puzzle to add
     * @see PDDReporter#report(Model, File)
     */
    public void add(final Puzzle puzzle) {
        if (this.codeUrlBuilder == null) {
            throw new IllegalStateException("Call setCodeUrlBuilder() first");
        }
        final String marker =
            this.PREFIX
            + this.SEPARATOR
            + puzzle.getHash()
            + this.SEPARATOR
            + puzzle.getIssueName();
        this.sud
            // find its UP-LINK
            .locate(puzzle.getIssueName())
            // find this particular puzzle under the LINK
            .locate(marker)
            // configure it
            .setEstimate(puzzle.getEstimate())
            .setSubject(puzzle.getSubject())
            .appendBody(
                "<a href=\""
                + this.codeUrlBuilder.build(
                    puzzle.getFile(),
                    puzzle.getRevision(),
                    puzzle.getLines()
                )
                + "\">"
                + puzzle.getFile()
                + "</a>"
            )
            .appendBody(puzzle.getBody());
        Logger.debug(
            this,
            "#add(%s): puzzle %s injected into WBS, marker: '%s'",
            puzzle.getClass().getName(),
            puzzle.getHash(),
            marker
        );
    }

    /**
     * Convert it to XML.
     * @return The xml created
     * @see PDDReporter#report(Model, File)
     * @todo #18 Would be nice to validate the output XML produced
     *           against pre-defined XML Schema.
     */
    public String toXml() {
        this.sud.optimize();
        final Writer writer = new StringWriter();
        try {
            final Marshaller mrsh = JAXBContext
                .newInstance(this.getClass()).createMarshaller();
            mrsh.setProperty(
                Marshaller.JAXB_FORMATTED_OUTPUT,
                Boolean.TRUE
            );
            mrsh.marshal(this, writer);
        } catch (javax.xml.bind.JAXBException ex) {
            throw RuntimeProblem.make(ex);
        }
        final String xml = writer.toString();
        Logger.info(
            this,
            "#toXml(): WBS converted to XML (%d bytes)",
            xml.length()
        );
        return xml;
    }

    /**
     * Get SuD.
     * @return The SUD
     */
    @XmlElement(name = "wp")
    public WorkPackage getSud() {
        return this.sud;
    }

}
